/*==============================================================================
    FILE:           user_audio_w_cdc.c

    DESCRIPTION:    Implements a linked list used to manage audio buffers passed between USB
                    and the CODECs.

    Copyright (c) 2018 Closed Loop Design, LLC

    This software is supplied "AS IS" without any warranties, express, implied
    or statutory, including but not limited to the implied warranties of fitness
    for purpose, satisfactory quality and non-infringement. Closed Loop Design LLC
    extends you a royalty-free right to use, reproduce and distribute this source
    file as well as executable files created using this source file for use with 
    Analog Devices SC5xx family processors only. Nothing else gives you 
    the right to use this source file.

==============================================================================*/
/*!
 * @file      ll_mgr.h
 * @brief     Audio buffer linked list manager.
 *
 * @details
 *            Implements a linked list used to manage audio buffers passed between USB
 *            and the CODECs.
 *
 *            Copyright (c) 2018 Closed Loop Design, LLC
 *
 *            This software is supplied "AS IS" without any warranties, express, implied
 *            or statutory, including but not limited to the implied warranties of fitness
 *            for purpose, satisfactory quality and non-infringement. Closed Loop Design LLC
 *            extends you a royalty-free right to use, reproduce and distribute this source
 *            file as well as executable files created using this source file for use with 
 *            Analog Devices SC5xx family processors only. Nothing else gives you 
 *            the right to use this source file.
 *
 */

#ifndef __LL_MGR__
#define __LL_MGR__

/**
 * Size of the data element of the linked list node.
 */
#define LL_MGR_AUDIO_DATA_SIZE      1024

/**
 * Linked List data node structure
 */
typedef struct LL_Mgr_Node LL_Mgr_Node;
struct LL_Mgr_Node
{
    LL_Mgr_Node * p_next;
    LL_Mgr_Node * p_prev;

    unsigned long size;
    unsigned char data[LL_MGR_AUDIO_DATA_SIZE]__attribute__ ((aligned (4)));
};

/**
 * Linked List Structure
 */
typedef struct
{
    LL_Mgr_Node * p_head;
    LL_Mgr_Node * p_tail;

    unsigned long num_elements;
} LL_Mgr_Data;


extern void ll_mgr_init (LL_Mgr_Data * p_ll);
extern void ll_mgr_insert (LL_Mgr_Data * p_ll, LL_Mgr_Node * p_node);
extern LL_Mgr_Node * ll_mgr_remove (LL_Mgr_Data * p_ll);
extern void ll_mgr_free_node (LL_Mgr_Node * p_node);
extern LL_Mgr_Node * ll_mgr_get_node (unsigned char clear_data);
extern void ll_mgr_reset (LL_Mgr_Data * p_ll);

#endif /* __LL_MGR__ */
